#!/bin/bash

#==============================================================================
# create configuration scripts (mainly in Shared/Config)
#==============================================================================

fun_configure_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export find_ini_config get_ini_value set_ini_value

[[ -n $fun_host_sourced ]] || . $(dirname $BASH_SOURCE)/fun-host.sh
export my_ips

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export find_host_section get_install_host
export find_required_packages update_required_packages

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# find_host_section
#..............................................................................
find_host_section() {
  local USAGE='
syntax: find_host_section INI_FILE
description: echoes section name [HOST] in a config file, HOST is:
             - [$(hostname -f)] if exist
             - else [$(hostname -s)] if exist
             - else [localhost] if exist
example: find_host_section install-epages.conf.sample
returns: success if found, else failure
'
  [[ $# != 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  local INI_FILE=$1

  for section in $(hostname -f) $(hostname -s) localhost ; do
     if grep -q "^\s*\[$section\]" $INI_FILE ; then
        echo $section ; return 0
     fi
  done
  return 1
}

#..............................................................................
# get_install_host
#..............................................................................
get_install_host() {
  local USAGE='
syntax: get_install_host EPAGES_INSTALL_CONF SECTION
description: echoes host of section [SECTION] in EPAGES_INSTALL_CONF:
  - EPAGES_INSTALL_CONF: [SECTION]:ip= if exist
  - else EPAGES_INSTALL_CONF: [SECTION]:hostname= if exist
  - else my IP if SECTION = localhost
  - else SECTION
'
  [[ $# != 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local EPAGES_INSTALL_CONF=$1
  [[ -f $EPAGES_INSTALL_CONF ]] || return

  local HOST=$(get_ini_value "$EPAGES_INSTALL_CONF" ip "$2")
  [[ -z $HOST ]] || { echo $HOST; return; }
  local HOST=$(get_ini_value "$EPAGES_INSTALL_CONF" hostname "$2")
  [[ -z $HOST ]] || { echo $HOST; return; }
  [[ $2 != localhost ]] || { my_ips | grep '\.' | head -1; return; }

  echo $2
}


#..............................................................................
# find_required_packages
#..............................................................................
find_required_packages() {
  local USAGE='
syntax: find_required_packages INSTALL_CONF [MYSQLD_CONF]
description: a) checks that everything is installed before patchhost can run.
  for each package in [hostname] checks if a key installed_package=1 exist.
  for example, if mysqld=1 is used then installed_mysqld=1 must exist.
  b) finds missing databases for specific packages. each installed database
  is added to installed_databases=comma,sep,list in [general definitions].
  for example, if j=1 is used then installed_databases= must contain epagesj.
arguments:
  INSTALL_CONF: install epages config files like install-epages.conf
  MYSQLD_CONF: install myslqd config file like mysqld/simple.conf
example: find_required_packages install-epages.conf mysqld/simple.conf
returns: success (0) - If all required packages are installed
         failure (1) - Echoes all missing required packages
'
  [[ $# = 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local INSTALL_CONF=$1
  local MYSQLD_CONF=$2
  local INI_CONFIG MYSECTION PATCHHOST SERVER_USED SERVER_INSTALLED

  INI_CONFIG=$(find_ini_config) || return 1
  MYSECTION=$(find_host_section $INSTALL_CONF) || return 0

  # Find required packages for patchhost.
  PATCHHOST=$(get_ini_value $INSTALL_CONF 'patchhost' $MYSECTION) || true

  if [[ -n $PATCHHOST && $PATCHHOST != 0 ]] ; then
    $INI_CONFIG -f $INSTALL_CONF -G | while read section ; do
      case $section in
	'general definitions') continue ;;
	'unix groups'|'unix users') continue ;;
	$(hostname -f)|$(hostname -s)|$MYSECTION) continue ;;
      esac

      for srv in appsrv ascsrv j mysqld reqsrv webconf mongo ; do
	SERVER_USED=$(get_ini_value $INSTALL_CONF $srv $section) || true
	[[ -n $SERVER_USED && $SERVER_USED != 0 ]] || continue

	SERVER_INSTALLED=$(get_ini_value $INSTALL_CONF installed_$srv $section) || true
	if [[ -z $SERVER_INSTALLED || $SERVER_INSTALLED = 0 ]] ; then
	  echo "Error: patchhost requires: $srv on $section"
	  touch /tmp/error.~$$~
	fi
      done
    done
  fi

  # INSTALLED_DB and MY_DB are comma-sep. lists of business units
  local DB_INSTALLED=$(get_ini_value $INSTALL_CONF installed_databases 'general definitions')
  local MY_DB='Site,Store,System,ASPoolDB,epagesj'

  if [[ -f $MYSQLD_CONF ]] ; then
    MY_DB=$($INI_CONFIG -f $MYSQLD_CONF -G | grep '\S' | grep -v ' ' | tr '\n' ',')
  elif [[ ${MYSQLD_CONF:-0} = 0 ]] ; then
    MY_DB=
  fi

  # add my own databases to DB_INSTALLED because they will be installed
  DB_INSTALLED="$DB_INSTALLED,$MY_DB"

  # Find required database for epages-j
  local EPAGES_J=$(get_ini_value $INSTALL_CONF 'j' $MYSECTION)
  if [[ ${EPAGES_J:-0} != 0 ]] ; then
    if ! echo $DB_INSTALLED | tr ',' '\n' | grep -q '^\s*epagesj\s*$' ; then
      echo "Error: package epages-j requires database: epagesj"
      touch /tmp/error.~$$~
    fi
  fi

  # Find required database for epages-reqsrv/epages-ascsrv
  local REQSRV=$(get_ini_value $INSTALL_CONF 'reqsrv' $MYSECTION)
  local ASCSRV=$(get_ini_value $INSTALL_CONF 'ascsrv' $MYSECTION)
  local PACKAGES_REQUIRE=
  if [[ ${REQSRV:-0} != 0 && ${ASCSRV:-0} != 0 ]] ; then
    PACKAGES_REQUIRE='s epages-reqsrv/epages-ascsrv require'
  elif [[ ${REQSRV:-0} != 0 ]] ; then
    PACKAGES_REQUIRE=' epages-reqsrv requires'
  elif [[ ${ASCSRV:-0} != 0 ]] ; then
    PACKAGES_REQUIRE=' epages-ascsrv requires'
  fi

  if [[ -n $PACKAGES_REQUIRE ]] ; then
    if ! echo $DB_INSTALLED | tr ',' '\n' | grep -q '^\s*ASPoolDB\s*$' ; then
      echo "Error: package$PACKAGES_REQUIRE database: ASPoolDB"
      touch /tmp/error.~$$~
    fi
  fi

  [[ ! -f /tmp/error.~$$~ ]] || { rm -f  /tmp/error.~$$~; return 1; }
  return 0
}

#..............................................................................
# update_required_packages
#..............................................................................
update_required_packages() {
  local USAGE='
syntax: update_required_packages INSTALL_CONF [MYSQLD_CONF]
description: a) updates installed_package=1 in my [hostname] for each
  installed package.
  b) adds installed databases to installed_databases=comma,sep,list in
  [general definitions].
arguments:
  INSTALL_CONF: install epages config files like install-epages.conf
  MYSQLD_CONF: install myslqd config file like mysqld/simple.conf
example: update_required_packages install-epages.conf mysqld/simple.conf
'
  [[ $# = 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local INSTALL_CONF=$1
  local MYSQLD_CONF=$2
  local MYSECTION MY_DB SERVER_USED

  MYSECTION=$(find_host_section $INSTALL_CONF) || return 0

  # set installed_package=1
  for srv in appsrv ascsrv j mysqld reqsrv webconf mongo ; do
    SERVER_USED=$(get_ini_value $INSTALL_CONF $srv $MYSECTION) || true
    [[ -n $SERVER_USED && $SERVER_USED != 0 ]] || continue
    set_ini_value --overwrite $INSTALL_CONF installed_$srv 1 $MYSECTION
  done

  # return if not databases were installed
  [[ ${MYSQLD_CONF:-0} != 0 ]] || return

  # add databases to installed_databases=
  local DB_INSTALLED=$(get_ini_value $INSTALL_CONF installed_databases 'general definitions')
  local NEW
  if [[ -f $MYSQLD_CONF ]] ; then
    NEW=$($INI_CONFIG -f $MYSQLD_CONF -G | grep '\S' | grep -v ' ' | tr '\n' ',')
  else
    NEW='Site,Store,System,ASPoolDB,epagesj'
  fi
  DB_INSTALLED=$(echo $DB_INSTALLED,$NEW | tr ',' '\n' | sort | uniq | grep . | tr '\n' ',')
  set_ini_value --overwrite $INSTALL_CONF installed_databases $DB_INSTALLED 'general definitions'

  return 0
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
