#!/bin/bash

#=======================================================================
# install-epages.sh
#=======================================================================

########################################################################
# usage
########################################################################

if [ ! -w /etc/passwd ] ; then
  echo "$0: only 'root' can execute this program"
  exit 1
fi

echo $BASH | grep -q '/bash$' && IS_BASH=1 || IS_BASH=
if [ -z "$BASH_VERSION" -o -z "$IS_BASH" ] ; then
  echo "$0: this script can only be executed by bash"
  exit 1
fi

LANG=C # use base language for standard output of some commands
LC_ALL=C

#-----------------------------------------------------------------------
# re-run myself putting output into log file
#-----------------------------------------------------------------------

[ -n "${BASH_SOURCE[0]}" ] && RUN=script || RUN=pipe

# no logfile for piped script or mock script
if ! [[ $@ =~ --(log|mock) ]] && [[ $RUN = script ]] ; then
  mkdir -pm 1777 /tmp/epages/install
  chmod 1777 /tmp/epages
  BN=${0##*/}
  LOGFILE=/tmp/epages/install/${BN%.sh}-$(date '+%Y.%m.%d-%H.%M.%S').log
  chmod 755 $0
  time $0 --log "$@" 2>&1 | tee $LOGFILE
  RESULT=${PIPESTATUS[0]:-0}
  echo -e "\nLOGFILE: $LOGFILE\n"
  exit $RESULT
fi

#------------------------------------------------------------------------------
# arguments
#------------------------------------------------------------------------------

usage() {
  [[ -n $1 ]] && echo -e "Error:\n  $@"
  echo "
Usage:
  /var/epages/install-epages.sh {--singlehost|--distributed ['PACKAGE_LIST']}
   [--fileserver [HOST]] [--version V] [--repo R] [--configfile FILE] [--mock]
  /var/epages/install-epages.sh --fileserver [HOST] [...]
  /var/epages/install-epages.sh --configfile FILE [...]

Usage (piped):
  [export EPAGES_INSTALLATION='{singlehost|distributed|PACKAGE_LIST}']
  [export EPAGES_INSTALLATION_MOCK=1]
  # uncustomized singlehost installation, for example:
  curl http://www.epages.com/repo/epages/install-epages.sh | bash

Environment (only used if script is piped):
  EPAGES_INSTALLATION : one of singlehost, distributed, 'PACKAGE_LIST' with
                        same meaning as in options; PACKAGE_LIST may contain
                        fileserver (for server) or fileserver=HOST (for client)
  Run uncustomized singlehost installation if EPAGES_INSTALLATION is unset.
  EPAGES_INSTALLATION_MOCK : perform a trial run with no changes made

Options:
  --version     : target version for ePages (default: latest version)
  --repo        : ePages repository (default: http://epages.com/repo/epages)
  --singlehost  : unpack and install ePages on single host installation
  --distributed : just unpack specific ePages packages on distributed install
  PACKAGE_LIST  : one/more of: appsrv/patchhost ascsrv reqsrv mysqld webconf j
                  mongo nlb epages (space-sep. list); default=epages: all
                  ePages packages but nlb; might be prefixed with minus:
                  'epages -mysqld' means all packages but mysqld
  --fileserver  : if HOST is used and is not own host, mount Shared/ from HOST
                  if HOST is unset or own host, share Shared/ for other hosts
  --configfile  : read configuration from FILE; options on command line always
                  have higher precedence than config file values; if option is
                  missing and /srv/epages/etc/sysconfig/install-epages.conf
                  exists; then use this; see example config in
                  /srv/epages/etc/sysconfig/install-epages.conf.sample
  --mock        : just --mock performs a trial run with no changes made,

Examples (single host installation):
  /var/epages/install-epages.sh --singlehost
  /var/epages/install-epages.sh --singlehost --version 6.17.34 --repo ftp://epages-software.de/repo/epages
  # this just unpacks but not starts ePages on a single host:
  /var/epages/install-epages.sh --distributed
  # this just installs epages-release:
  /var/epages/install-epages.sh --distributed -epages

Example A (distributed installation):
  # first create the share on one HOST:
  /var/epages/install-epages.sh --fileserver --distributed mysqld
  # then mount the share on other hosts:
  /var/epages/install-epages.sh --fileserver HOST --distributed 'epages -mysqld'
  # then on each involved host start ePages packages (*finally* on appsrv)
  /etc/init.d/epages6 start

Example B (distributed installation):
  # first create the share on one HOST:
  /var/epages/install-epages.sh --fileserver --distributed appsrv
  # then mount the share on other hosts:
  /var/epages/install-epages.sh --fileserver HOST --distributed appsrv
  /var/epages/install-epages.sh --fileserver HOST --distributed 'ascsrv reqsrv webconf'
  /var/epages/install-epages.sh --fileserver HOST --distributed j
  /var/epages/install-epages.sh --fileserver HOST --distributed mysqld
  # then on each involved host start ePages packages (*finally* on appsrv)
  /etc/init.d/epages6 start

Examples (with configuration file):
  /var/epages/install-epages.sh --version 6.17.34 --configfile /tmp/install-epages.conf

Examples (piped):
  # uncustomized singleserver installation via pipe:
  curl http://www.epages.com/repo/epages/install-epages.sh | bash
  # distributed installation via pipe:
  # first create the share and install mysqld and j on one HOST:
  export EPAGES_INSTALLATION='fileserver mysqld j'
  curl http://www.epages.com/repo/epages/install-epages.sh | bash
  # then mount the share and install everything else on other host:
  export EPAGES_INSTALLATION='fileserver=HOST epages -mysqld -j'
  curl http://www.epages.com/repo/epages/install-epages.sh | bash
"
  exit 2
}

getarg() {
  # getarg -0 ... => no arguments allowed
  # getarg -1 ... => exactly 1 argument allowed
  # getarg ...    => 0 or 1 arguments allowed
  local req
  case $1 in
    -[01]) req=$1; shift; ;;
  esac
  if [[ ${2:---} =~ ^-- && $req = -1 ]] ; then
    # after option comes nothing or other option
    usage "Option requires 1 argument: $1" 1>&2
  elif [[ ${2:---} =~ ^-- ]] ; then
    # after option comes nothing or other option
    echo " "; return 1
  elif [[ $req = -0 ]] ; then
    # after option comes 1 argument
    usage "Option requires 0 arguments: $1" 1>&2
  elif ! [[ ${3:---} =~ ^-- ]] ; then
    # after option comes more than 1 argument
    usage "Option requires 0 or 1 arguments: $1 $2 $3" 1>&2
  else
    # after option comes 1 argument
    echo "$2"; return 0
  fi
}

unset SINGLEHOST DISTRIBUTED FILESERVER VERSION REPO LOG CONFIGFILE MOCK PATCHHOST

# use EPAGES_INSTALLATION/EPAGES_INSTALLATION_MOCK if script is piped
if [[ $RUN = pipe ]] ; then
  case $EPAGES_INSTALLATION in
    singlehost) SINGLEHOST=1 ;;
    distributed) DISTRIBUTED=' ' ;;
    *[a-z]*) for i in $EPAGES_INSTALLATION ; do
               # patchhost is an appsrv
               [[ $i != patchhost ]] || { PATCHHOST=1 ; i=appsrv; }
               if [[ $i =~ fileserver= ]] ; then FILESERVER=${i#*=}
               elif [[ $i =~ fileserver ]] ; then FILESERVER=' '
               else DISTRIBUTED="$DISTRIBUTED $i" ; fi
             done ; DISTRIBUTED="${DISTRIBUTED# }" ;;
    # run uncustomized singlehost if EPAGES_INSTALLATION is unset:
    # curl http://www.epages.com/repo/epages/install-epages.sh | bash
    *) SINGLEHOST=1 ;;
  esac
  MOCK=$EPAGES_INSTALLATION_MOCK
fi

while [[ $# -gt 0 ]] ; do
  case "$1" in
    --singlehost) SINGLEHOST=$(getarg -0 "$@"); RET=0; shift ;;
    --distributed) DISTRIBUTED=$(getarg "$@") && shift; RET=$?; shift ;;
    --fileserver) FILESERVER=$(getarg "$@") && shift; RET=$?; shift ;;
    --configfile) CONFIGFILE=$(getarg -1 "$@"); RET=$?; shift; shift ;;
    --version) VERSION=$(getarg -1 "$@"); RET=$?; shift; shift ;;
    --repo) REPO=$(getarg -1 "$@"); RET=$?; shift; shift ;;
    --log) LOG=$(getarg -0 "$@"); RET=0; shift ;;
    --mock) MOCK=$(getarg "$@") && shift; RET=$?; shift ;;
    --help) usage ;;
    *) usage "Void option: $1" ;;
  esac
  [[ $RET -gt 1 ]] && exit 1
done

# no other arguments allowed
[[ -n $1 ]] && usage "Argument(s) not allowed: $@"

if [[ ( -n $SINGLEHOST && -n $DISTRIBUTED ) || -z "$CONFIGFILE$FILESERVER$SINGLEHOST$DISTRIBUTED" ]]
then
  echo "$0: choose either --singlehost or --distributed"
  exit 1
fi

# install everything if --distributed has no argument
[[ $DISTRIBUTED = " " ]] && DISTRIBUTED=epages

if [[ $DISTRIBUTED != ${DISTRIBUTED//patchhost/appsrv} ]] ; then
  # patchhost is an appsrv
  DISTRIBUTED=${DISTRIBUTED//patchhost/appsrv}
  PATCHHOST=1
fi

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  [[ -n $MOCK ]] && echo "$@" || echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

########################################################################
# define variables
########################################################################

#-----------------------------------------------------------------------
# find OS
#-----------------------------------------------------------------------

unset OS
if [[ -f /etc/debian_version ]] ; then
  [[ -n $MOCK ]] || ECHO $LINENO "Debian found: /etc/debian_version -> $(cat /etc/debian_version)"
  OS=debian
  INSTALLER=apt-get
  sed -ri 's,^(\s*deb\s+cdrom),# \1,' \
  /etc/apt/sources.list 2>/dev/null || true
elif [[ -f /etc/redhat-release ]] ; then
  [[ -n $MOCK ]] || ECHO $LINENO "Redhat found: /etc/redhat-release -> $(cat /etc/redhat-release)"
  OS=redhat
  INSTALLER=yum
else
  echo "$0: operating system not supported: $(uname -a)"
  exit 1
fi

UNAME_R=$(uname -r)
cnt=0
for i in ${UNAME_R//[-.]/ } ; do
  KERNEL=$KERNEL.$i
  cnt=$(($cnt + 1))
  [[ $cnt -ge 3 ]] && break
done

KERNEL=${KERNEL#.}
[[ $OS = debian ]] && KERNEL=${KERNEL}d || KERNEL=${KERNEL}r

#-----------------------------------------------------------------------
# check version
#-----------------------------------------------------------------------

if [[ -n $VERSION ]] ; then
  if [[ -n ${VERSION//[0-9.]/} ]] ; then
    echo "$0: version must contain only numbers and dots, not: --version $VERSION"
    exit 1
  fi
  case $KERNEL in
    *d) grep -q '^7' /etc/debian_version && REQUIRED=6.17.5 || REQUIRED=6.17.29 ;;
    2.*r) REQUIRED=6.12.0 ;;
    3.*r) REQUIRED=6.17.28 ;;
  esac
  [[ -z $MOCK ]] || REQUIRED=6.12.0
  if [[ $(echo -e "$VERSION\n$REQUIRED" | sort -V | head -1) != $REQUIRED ]] ; then
    echo "$0: version must be at least $REQUIRED, not: --version $VERSION"
    exit 1
  fi
  if [[ $(echo -e "$VERSION\n6.17.0" | sort -V | head -1) != 6.17.0 ]] ; then
    # legacy versions < 6.17.0
    REPO=${REPO:-ftp://ftp.epages.com/pub/epages/build/repo/$VERSION/latest}
  fi
fi

#-----------------------------------------------------------------------
# test configfile
#-----------------------------------------------------------------------

if [[ -z $CONFIGFILE && -f /srv/epages/etc/sysconfig/install-epages.conf ]] ; then
  # use default config file if exists and --configfile is not used
  CONFIGFILE=/srv/epages/etc/sysconfig/install-epages.conf
fi
if [[ -n $CONFIGFILE ]] ; then
  [[ -f $CONFIGFILE ]] || mkdir -m 755 -p ${CONFIGFILE%/*}
  CONFIGFILE=$(readlink -m "$CONFIGFILE")
  if ! touch $CONFIGFILE ; then
    echo "$0: cannot write $CONFIGFILE"
    exit 1
  fi
  chmod 600 $CONFIGFILE
fi

#-----------------------------------------------------------------------
# declare variables
#-----------------------------------------------------------------------
# <key>-<value> pairs are used to set EPAGES_INSTALL_<key>=<value> in /etc/sysconfig/epages6
declare -A EPAGES_INSTALL
# MYSECTION is either $(hostname -f) if exist, else $(hostname -s) if exist, else localhost
unset MYSECTION
# EPAGES_CONF_NEW contains value of key epages-conf in [general definitions]
unset EPAGES_CONF_NEW
# MYSQL contains [general definitions] of mysqld-root-password and my-cnf
declare -A MYSQL
# MYHOST contains <key>-<value> pairs of section [$MYSECTION] in install-epages.conf
declare -A MYHOST
# MYPACKAGES contains packages to install
declare -A MYPACKAGES
# mapping of section names with key hostname in install-epages.conf
declare -A hostname
# mapping of section names with key ip in install-epages.conf
declare -A ip
# is set then other hosts than my host are defined in install-epages.conf
unset OTHERHOSTS
# contains section name of section with patchhost=1 in install-epages.conf
unset PATCHHOST_SECTION
# contains section name of section with fileserver=server in install-epages.conf
unset FILESERVER_SECTION

#-----------------------------------------------------------------------
# set_patchhost_section SECTION VALUE
# - if VALUE is unset or 0 - do nothing
# - if PATCHHOST_SECTION is SECTION - do nothing
# - if PATCHHOST_SECTION is empty - set it to SECTION
# - if PATCHHOST_SECTION is otherwise set - throw error (may be defined only once)
#-----------------------------------------------------------------------
set_patchhost_section() {
  local SECTION=$1
  local VALUE=$2
  [[ ${VALUE:-0} = 0 ]] && return
  [[ $PATCHHOST_SECTION = $SECTION ]] && return
  [[ -z $PATCHHOST_SECTION ]] && { PATCHHOST_SECTION=$SECTION; return; }
  echo "$0: patchhost=1 can be set only once in: $CONFIGFILE"
  exit 1
}

#-----------------------------------------------------------------------
# set_fileserver_section SECTION
# - if VALUE is not 'server' - do nothing
# - if FILESERVER_SECTION is SECTION - do nothing
# - if FILESERVER_SECTION is empty - set it to SECTION
# - if FILESERVER_SECTION is otherwise set - throw error (may be defined only once)
#-----------------------------------------------------------------------
set_fileserver_section() {
  local SECTION=$1
  local VALUE=$2
  [[ $VALUE = server ]] || return
  [[ $FILESERVER_SECTION = $SECTION ]] && return
  [[ -z $FILESERVER_SECTION ]] && { FILESERVER_SECTION=$SECTION; return; }
  echo "$0: fileserver=server can be set only once in: $CONFIGFILE"
  exit 1
}

#-----------------------------------------------------------------------
# read_install_epages_conf CONFIGFILE
# - fill above declared variables with values from CONFIGFILE
#-----------------------------------------------------------------------

read_install_epages_conf() {
  local CONFIGFILE=$1
  local SECTION

  while IFS== read key value ; do
    # delete leading and trailing spaces
    key=$(echo "$key" | sed -r 's,^\s*,,;s,\s*$,,')
    value=$(echo "$value" | sed -r 's,^\s*,,;s,\s*$,,')
    # ignore empty lines or comments
    [[ -z $key || $key =~ ^# || $key =~ ^\; ]] && continue
    # section found
    if [[ ${key#[} != $key ]] ; then
      SECTION=${key#[} ; SECTION=${SECTION%]}
      unset SKIP_MYSECTION
      case $SECTION in
        # my section with highest priority is full qualified domain name
        $(hostname -f)) MYSECTION=$SECTION ; SKIP_MYSECTION=1 ;;
        # my section with 2nd highest priority is short domain name
        $(hostname -s)) [[ $MYSECTION = $(hostname -f) ]] || MYSECTION=$SECTION ; SKIP_MYSECTION=1 ;;
        # my section with lowest priority is localhost
        localhost) MYSECTION=${MYSECTION:-localhost} ; SKIP_MYSECTION=1 ;;
      esac
      if [[ -n $SKIP_MYSECTION && $MYSECTION = $SECTION ]] ; then
        # my section with highest priority (up to now) found -> re-fill MYHOST/MYPACKAGES
        MYHOST=();  MYPACKAGES=(); SKIP_MYSECTION=
      fi
      continue
    fi

    # unknown section for key=value pair
    [[ -z $SECTION ]] && continue

    # skip my section if section with higher priority already found
    [[ -n $SKIP_MYSECTION ]] && continue

    # host unspecific sections
    if [[ $SECTION = 'general definitions' ]] ; then
      case $key in
        version) VERSION=${VERSION:-$value} ;;
        repo) REPO=${REPO:-$value} ;;
        mysqld-root-password|my-cnf) MYSQL["$key"]=$value ;;
        epages-conf) EPAGES_CONF_NEW=${EPAGES_CONF_NEW:-$value} ;;
      esac
    elif [[ $SECTION = 'unix groups' && -n $value ]] ; then
      if [[ -n ${value//[0-9]/} ]] ; then
        echo "$0: $key=$value: group ID must be a number"
        exit 1
      fi
      case $key in
        epages)  EPAGES_INSTALL[EP_GID]=$value ;;
        ep_web)  EPAGES_INSTALL[WS_GID]=$value ;;
      esac
    elif [[ $SECTION = 'unix users' && -n $value ]] ; then
      if [[ -n ${value//[0-9]/} ]] ; then
        echo "$0: $key=$value: user ID must be a number"
        exit 1
      fi
      case $key in
        ep_appl)  EPAGES_INSTALL[AS_UID]=$value ;;
        eprunapp) EPAGES_INSTALL[AS_RUNUID]=$value ;;
        epagesj)  EPAGES_INSTALL[EPAGESJ_UID]=$value ;;
        ep_web)  EPAGES_INSTALL[WS_UID]=$value ;;
        eprunweb) EPAGES_INSTALL[WS_RUNUID]=$value ;;
      esac

    # my host section
    elif [[ $SECTION = $MYSECTION ]] ; then
      MYHOST[$key]=$value
      case $key in
        appsrv|ascsrv|reqsrv|mysqld|webconf|j|mongo|nlb)
        [[ ${value:-0} = 0 ]] || MYPACKAGES[$key]=$value ;;
        ip) ip[$SECTION]=$value ;;
        hostname) hostname[$SECTION]=$value ;;
        fileserver) set_fileserver_section $SECTION $value ;;
        patchhost) set_patchhost_section $SECTION $value ;;
      esac

    # other host sections
    elif ! [[ $SECTION =~ ' ' ]] ; then
      [[ ${value:-0} = 0 ]] && continue
      OTHERHOSTS=1
      case $key in
        ip) ip[$SECTION]=$value ;;
        hostname) hostname[$SECTION]=$value ;;
        fileserver) set_fileserver_section $SECTION $value ;;
        patchhost) set_patchhost_section $SECTION $value ;;
        appsrv) EPAGES_INSTALL[AS_HOST]=$SECTION ;;
        ascsrv) EPAGES_INSTALL[RR_HOST]=$SECTION ;;
        reqsrv) EPAGES_INSTALL[RR_HOST]=$SECTION ;;
        mysqld) EPAGES_INSTALL[MYSQL_HOST]=$SECTION ;;
        webconf) EPAGES_INSTALL[WS_HOST]=$SECTION ;;
        j) EPAGES_INSTALL[EPAGESJ_HOST]=$SECTION ;;
        mongo) EPAGES_INSTALL[MONGO_HOST]=$SECTION ;;
      esac
    fi
  done < "$CONFIGFILE"
}

#-----------------------------------------------------------------------
# read install-epages.conf
#-----------------------------------------------------------------------

if [[ -f $CONFIGFILE ]] ; then
  ECHO $LINENO read_install_epages_conf "$CONFIGFILE"
  read_install_epages_conf "$CONFIGFILE"
fi

#-----------------------------------------------------------------------
# check if version has some packages
#-----------------------------------------------------------------------

unset HAS_J HAS_ASCSRV

if [[ -z $VERSION || $(echo -e "$VERSION\n6.17.5" | sort -V | head -1) = 6.17.5  ]] ; then
   HAS_J=j; HAS_ASCSRV=ascsrv
elif [[ $(echo -e "$VERSION\n6.14.0" | sort -V | head -1) = 6.14.0 ]] ; then
   HAS_J=j
fi

#-----------------------------------------------------------------------
# define SINGLEHOST or DISTRIBUTED according to install-epages.conf
#-----------------------------------------------------------------------

set_patchhost_section "$MYSECTION" ${MYHOST[patchhost]}
[[ ${MYHOST[patchhost]:-0} = 0 ]] || PATCHHOST=1

if [[ -z "$SINGLEHOST$DISTRIBUTED" ]] ; then
  # distributed installation if:
  # - config file is used
  [[ -f $CONFIGFILE ]] && DISTRIBUTED=1
  # - other hosts exist
  [[ -z $OTHERHOSTS ]] || DISTRIBUTED=1
  # - my host is not patchhost
  [[ -n $PATCHHOST ]] || DISTRIBUTED=1
  # - my host uses fileserver
  [[ ${MYHOST[fileserver]:-0} = 0 ]] || DISTRIBUTED=1
  # - my host uses nlb
  [[ -z ${MYPACKAGES[nlb]} ]] || DISTRIBUTED=1
  # - my host won't install all packages
  for package in appsrv reqsrv $HAS_ASCSRV mysqld webconf $HAS_J ; do
    [[ -n ${MYPACKAGES[$package]} ]] && continue
    DISTRIBUTED=1 ; break
  done

  # else singlehost installation
  [[ -z $DISTRIBUTED ]] && SINGLEHOST=1

  DISTRIBUTED=${DISTRIBUTED:+${!MYPACKAGES[@]}}
fi

#-----------------------------------------------------------------------
# define FILESERVER according to install-epages.conf
#-----------------------------------------------------------------------

if [[ -z $FILESERVER && ${MYHOST[fileserver]} = server ]] ; then
  set_fileserver_section "$MYSECTION" server
  FILESERVER=' '
elif [[ -z "$FILESERVER$FILESERVER_SECTION" && ${MYHOST[fileserver]} = client ]] ; then
  echo "$0: no fileserver=server for [$MYSECTION]:fileserver=client of $CONFIGFILE"
  exit 1
elif [[ -z $FILESERVER && ${MYHOST[fileserver]} = client ]] ; then
  FILESERVER=${hostname[$FILESERVER_SECTION]}
  FILESERVER=${FILESERVER:-${ip[$FILESERVER_SECTION]}}
  FILESERVER=${FILESERVER:-$FILESERVER_SECTION}
fi

if [[ -z "$FILESERVER$SINGLEHOST$DISTRIBUTED" ]] ; then
  echo "$0: no servers found in my sections of $CONFIGFILE"
  exit 1
fi

# install nothing if just --fileserver was used
[[ -z "$SINGLEHOST$DISTRIBUTED" ]] && DISTRIBUTED=-epages

[[ -n $CONFIGFILE ]] && ECHO $LINENO CONFIGFILE=$CONFIGFILE
[[ -n $MYSECTION ]] && ECHO $LINENO MYSECTION=$MYSECTION
[[ -n $FILESERVER ]] && ECHO $LINENO FILESERVER=$FILESERVER
[[ -n $SINGLEHOST ]] && ECHO $LINENO SINGLEHOST=$SINGLEHOST
[[ -n $DISTRIBUTED ]] && ECHO $LINENO DISTRIBUTED=$DISTRIBUTED
[[ -n $VERSION ]] && ECHO $LINENO VERSION=$VERSION
[[ -n $REPO ]] && ECHO $LINENO REPO=$REPO

#-----------------------------------------------------------------------
# check_install
#-----------------------------------------------------------------------

check_install() {
  local PACKAGE=$1
  local RESULT=${2:-UNKNOWN}

  if [[ $RESULT != FAILURE ]] ; then
    if [[ -f /tmp/epages/epages-$PACKAGE/post.failure ]] ; then
      RESULT=FAILURE
    elif [[ ! -f /tmp/epages/epages-$PACKAGE/post.success ]] ; then
      RESULT=UNKNOWN
    fi
  fi
  echo "Result from $INSTALLER -y install epages-$PACKAGE: $RESULT"
  [[ $RESULT = FAILURE ]] && return 1 || return 0
}

#-----------------------------------------------------------------------
# exit immediately if a simple command exits with a non-zero status
#-----------------------------------------------------------------------

set -e

########################################################################
# install 3rd party packages
########################################################################

#-----------------------------------------------------------------------
# install curl
#-----------------------------------------------------------------------

if ! type curl >/dev/null && [[ -z $MOCK ]] ; then
  ECHO $LINENO $INSTALLER -y install curl
  $INSTALLER -y install curl
fi

#-----------------------------------------------------------------------
# install gpgme (CentOS 6)
#-----------------------------------------------------------------------

if cat /etc/redhat-release 2>/dev/null | grep -q 'release 6' ; then
  if ! rpm --quiet -q gpgme && [[ -z $MOCK ]] ; then
    ECHO $LINENO yum -y install gpgme
    yum -y install gpgme
  fi
fi

#-----------------------------------------------------------------------
# update nss-softokn (CentOS 6)
#-----------------------------------------------------------------------

if cat /etc/redhat-release 2>/dev/null | grep -q 'release 6' ; then
  for i in i686 x86_64 ; do
    [[ -z $MOCK ]] || continue
    rpm --quiet -q nss-softokn.$i && has_softokn=1 || has_softokn=0
    rpm --quiet -q nss-softokn-freebl.$i && has_freebl=1 || has_freebl=0
    if [[ $has_freebl != $has_softokn ]] ; then
      ECHO $LINENO yum -y install nss-softokn nss-softokn.i686 nss-softokn-freebl nss-softokn-freebl.i686
      yum -y install nss-softokn nss-softokn.i686 nss-softokn-freebl nss-softokn-freebl.i686
      break
    fi
  done
fi

#-----------------------------------------------------------------------
# install perl-Data-Dumper (CentOS 7)
#-----------------------------------------------------------------------

if cat /etc/redhat-release 2>/dev/null | grep -q 'release 7' && [[ -z $MOCK ]] ; then
  ECHO $LINENO yum -y install perl-Data-Dumper
  yum -y install perl-Data-Dumper
fi

#-----------------------------------------------------------------------
# install openssh
#-----------------------------------------------------------------------

if ! type ssh-keygen >/dev/null && [[ -z $MOCK ]] ; then
  ECHO $LINENO $INSTALLER -y install openssh
  $INSTALLER -y install openssh
fi

#-----------------------------------------------------------------------
# remove epages packages marked as rc (Debian)
#-----------------------------------------------------------------------

if [[ $OS = debian && -z $MOCK ]] ; then
  dpkg --list | awk '/^rc.*epages/{print $2}' | xargs dpkg --purge 2>/dev/null || true
fi

########################################################################
# install epages-release
########################################################################

REPO_URL=${REPO:-http://epages.com/repo/epages}

unset INSTALLED_RELEASE NEW_RELEASE
if [[ $OS = debian && -z $MOCK ]] ; then
  ECHO $LINENO curl -sLkO $REPO_URL/apt/epages-release_all.deb
  curl -sLkO $REPO_URL/apt/epages-release_all.deb
  NEW_RELEASE=$(dpkg-deb -f epages-release_all.deb Version) || true
  INSTALLED_RELEASE=$(dpkg-query --show -f '${Version}\n' epages-release 2>/dev/null) || true
  if dpkg --compare-versions ${INSTALLED_RELEASE:-0} lt ${NEW_RELEASE:-0.1} ; then
    ECHO $LINENO dpkg -i epages-release_all.deb
    dpkg -i epages-release_all.deb
  fi
elif [[ -z $MOCK ]] ; then
  NEW_RELEASE=$(rpm --quiet -qp $REPO_URL/yum/epages-release.noarch.rpm --qf '[%{VERSION}-%{RELEASE}\n]') || true
  INSTALLED_RELEASE=$(rpm --quiet -q epages-release --qf '[%{VERSION}-%{RELEASE}\n]' 2>/dev/null) || true
  if [[ $(echo -e "${NEW_RELEASE:-0}\n$INSTALLED_RELEASE" | sort -V | tail -1) != $INSTALLED_RELEASE ]] ; then
    ECHO $LINENO rpm -hUv $REPO_URL/yum/epages-release.noarch.rpm
    # ENABLE_REPO for legacy
    ENABLE_REPO='epages epages-kernel epages-setup' rpm -hUv $REPO_URL/yum/epages-release.noarch.rpm || true
  fi
fi

#-----------------------------------------------------------------------
# load functions we need
#-----------------------------------------------------------------------

VAR_EPAGES=$(cd $(dirname $0) && pwd)
if [[ ! -f $VAR_EPAGES/ini-config.sh ]] ; then
  [[ -f /var/epages/ini-config.sh ]] && VAR_EPAGES=/var/epages || VAR_EPAGES=/var/tmp
fi
ECHO $LINENO VAR_EPAGES=$VAR_EPAGES
chmod 755 $VAR_EPAGES/*.sh

INI_CONFIG=$VAR_EPAGES/ini-config.sh
SYSCONFIG=/etc/sysconfig/epages6

set +e
for i in arch configure epages account repoedit pkg ; do
  [[ -f $VAR_EPAGES/functions/fun-$i.sh ]] || continue
  ECHO $LINENO . $VAR_EPAGES/functions/fun-$i.sh
  . $VAR_EPAGES/functions/fun-$i.sh >/dev/null
done
set -e

#-----------------------------------------------------------------------
# update epages repository
#-----------------------------------------------------------------------

if  [[ $OS = redhat && -z $MOCK ]] && type enable_repo >/dev/null ; then
  ECHO $LINENO enable_repo -e 1 arch base setup
  enable_repo -e 1 arch base setup >/dev/null || true
fi
if [[ $OS = debian && -z $MOCK ]] && type enable_apt_repo >/dev/null ; then
  ECHO $LINENO enable_apt_repo 1 epages-setup
  enable_apt_repo 1 epages-setup >/dev/null || true
fi

if [[ -n "$REPO$VERSION$MOCK" && -f $VAR_EPAGES/SetRepository.sh ]] ; then

  VERSION_ARG=${VERSION:+-b $VERSION}
  REPO_ARG=${REPO:+-r $REPO_URL}
  if [[ -n "$VERSION_ARG$REPO_ARG" ]] ; then
    ECHO $LINENO $VAR_EPAGES/SetRepository.sh $VERSION_ARG $REPO_ARG
    [[ -n $MOCK ]] || $VAR_EPAGES/SetRepository.sh $VERSION_ARG $REPO_ARG
  fi

elif [[ -z $MOCK && ! -f $VAR_EPAGES/SetRepository.sh ]] ; then

  # legacy repository:
  RPMS=ftp://epages-software.de/pub/products/RPMS
  # new repository:
  REPO=${REPO:-ftp://ftp.epages.com/pub/epages/build/repo/$VERSION/latest}
  for i in $(ls -d /etc/yum.repos.d/epages*.repo 2>/dev/null) ; do
    ECHO $LINENO sed $i
    # ignore mirrors
    sed -ri 's,^(mirrorlist),#\1,' $i
    # ftp://epages-software.de/pub/products/RPMS/6.10-setup
    # -> ftp://ftp.epages.com/pub/epages/build/repo/$VERSION/latest/yum/base/2.6.32r
    sed -ri "s,$RPMS/6[^/]+-setup$,$REPO/yum/base/$KERNEL," $i
    # ftp://epages-software.de/pub/products/RPMS/6.14.2/$basearch
    # -> ftp://ftp.epages.com/pub/epages/build/repo/$VERSION/latest/yum/6.14.2/$basearch
    sed -ri "s,$RPMS/6[^/]+,$REPO/yum," $i
    # ftp://epages-software.de/pub/products/RPMS/kernel/2.6.32r
    # -> ftp://ftp.epages.com/pub/epages/build/repo/kernel/2.6.32r
    sed -ri "s,$RPMS/kernel/(x86_64/)?,$REPO/kernel/," $i
  done
  ECHO $LINENO yum clean metadata
  yum clean metadata

elif [[ $OS = debian ]] && dpkg --compare-versions ${INSTALLED_RELEASE:-0} lt ${NEW_RELEASE:-0} ; then

  ECHO $LINENO apt-get -q -q -y update
  [[ -n $MOCK ]] || apt-get -q -q -y update || true

fi

if [[ $OS = debian ]] && dpkg --compare-versions ${INSTALLED_RELEASE:-0} lt ${NEW_RELEASE:-0} ; then
  ECHO $LINENO apt-get -y upgrade
  [[ -n $MOCK ]] || apt-get -y upgrade || true
fi

########################################################################
# mount/share epages file system
########################################################################

if [[ -n ${FILESERVER// /} && -f $VAR_EPAGES/epages-fs.sh ]] ; then
  # mount $FILESERVER share
  ECHO $LINENO $VAR_EPAGES/epages-fs.sh -mount $FILESERVER
  [[ -n $MOCK ]] || $VAR_EPAGES/epages-fs.sh -mount $FILESERVER
elif [[ -n $FILESERVER && -f $VAR_EPAGES/epages-fs.sh ]] ; then
  # share $FILESERVER mount
  ECHO $LINENO $VAR_EPAGES/epages-fs.sh -share
  [[ -n $MOCK ]] || $VAR_EPAGES/epages-fs.sh -share
fi

########################################################################
# configuration files
########################################################################

#-----------------------------------------------------------------------
# define what packages to install
#-----------------------------------------------------------------------

unset INCLUDE EXCLUDE
declare -A INCLUDE EXCLUDE

# define what packages to INCLUDE and what packages to EXCLUDE
for package in $DISTRIBUTED ; do
  case $package in
    # do not install if package starts with -
    -*) EXCLUDE[${package#-}]=${package#-} ;;
    # otherwise install
    *) INCLUDE[$package]=$package ;;
  esac
done

# replace epages by any epages package
if [[ -n ${INCLUDE[epages]} ]] ; then
  unset INCLUDE[epages]
  for package in appsrv reqsrv mysqld webconf ; do
    INCLUDE[$package]=$package
  done
  # legacy: epages-j was introduced with 6.14.0
  [[ -z $HAS_J ]] || INCLUDE[j]=j
  # legacy: epages-ascsrv was introduced with 6.17.5
  [[ -z $HAS_ASCSRV ]] || INCLUDE[ascsrv]=ascsrv
fi

# drop any package in EXCLUDE from INCLUDE
unset SKIP_EPAGES
for package in ${!EXCLUDE[@]} ; do
  case $package in
    epages) SKIP_EPAGES=1 ;;
    *) unset INCLUDE[$package] ;;
  esac
done

# do not install any package (but nlb) if -epages was set
if [[ -n $SKIP_EPAGES && -n ${INCLUDE[nlb]} ]] ; then
  INCLUDE=()
  INCLUDE[nlb]=nlb
elif [[ -n $SKIP_EPAGES ]] ; then
  INCLUDE=()
fi

# mongo is installed by j
[[ -z ${INCLUDE[j]} ]] || unset INCLUDE[mongo]

#-----------------------------------------------------------------------
# overwrite _HOST keys in EPAGES_INSTALL with MYHOST values
#-----------------------------------------------------------------------

host=${MYSECTION:-$(hostname -f)}
for package in ${!INCLUDE[@]} ; do
  case $package in
    appsrv) EPAGES_INSTALL[AS_HOST]=$host ;;
    ascsrv) EPAGES_INSTALL[RR_HOST]=$host ;;
    reqsrv) EPAGES_INSTALL[RR_HOST]=$host ;;
    mysqld) EPAGES_INSTALL[MYSQL_HOST]=$host ;;
    webconf) EPAGES_INSTALL[WS_HOST]=$host ;;
    j) EPAGES_INSTALL[EPAGESJ_HOST]=$host ;;
    mongo) EPAGES_INSTALL[MONGO_HOST]=$host ;;
    nlb) true ;;
    *) echo "$0: $package: not a valid ePages package name" ; exit 1 ;;
  esac
done

#-----------------------------------------------------------------------
# update _HOST and _HOSTIP keys in EPAGES_INSTALL
#-----------------------------------------------------------------------

for key in "${!EPAGES_INSTALL[@]}" ; do
  # key must be _HOST
  [[ $key =~ _HOST$ ]] || continue
  value=${EPAGES_INSTALL[$key]}
  if [[ -n ${hostname[$value]} ]] ; then
    host=${hostname[$value]}
  elif [[ -n ${ip[$value]} ]] ; then
    host=${ip[$value]}
  elif [[ $value = localhost ]] ; then
    # if hostname= empty and ip= empty and section is [localhost] then use FQDN
    host=$(hostname -f)
  else
    host=$value
  fi
  # replace section name by its hostname/ip key
  EPAGES_INSTALL[$key]=$host
  case $key in
    AS_HOST|RR_HOST|WS_HOST)
      # add *_HOSTIP to *_HOST
      [[ -z ${ip[$value]} ]] || EPAGES_INSTALL["${key}IP"]=${ip[$value]} ;;
    MYSQL_HOST)
      # add MYSQL_CONNECTION
      [[ -z ${ip[$value]} ]] || EPAGES_INSTALL[MYSQL_CONNECTION]=${ip[$value]} ;;
    EPAGESJ_HOST)
      for server in MONGO_HOST SOLR_HOST SOLRSLAVE_HOST ; do
        # set server to EPAGESJ_HOST unless defined
        EPAGES_INSTALL[$server]=${EPAGES_INSTALL[$server]:-${EPAGES_INSTALL[$key]}}
      done
  esac
done

#-----------------------------------------------------------------------
# add absolute_path and get_ini_value unless already known
#-----------------------------------------------------------------------

if ! type absolute_path >/dev/null 2>&1 ; then
  # see documentation of absolute_path in functions/fun-arch.sh
  absolute_path () {
    local FILE=$1
    local RELATIVE_TO=$2
    if [[ ! $FILE =~ ^/ && -n $RELATIVE_TO ]] ; then
      RELATIVE_TO=$(readlink -m $RELATIVE_TO)
      FILE=${RELATIVE_TO%/*}/$FILE
    fi
    readlink -m $FILE
    [[ -f $FILE ]] && return 0 || return 1
  }
fi

if ! type get_ini_value >/dev/null 2>&1 ; then
  # see documentation of get_ini_value in functions/fun-epages.sh
  get_ini_value() {
    $INI_CONFIG -f "$1" -s "$3" -G | egrep -q "^$2\s*=" || return 1
    $INI_CONFIG -f "$1" -s "$3" -k "$2" -G
    return 0
  }
fi

#-----------------------------------------------------------------------
# if we install MySQL server, set MySQL variables (unless unset)
#-----------------------------------------------------------------------

unset ${!INSTALL_EPAGES_*}
unset MYSQLD_CONFIGFILE EPAGES_ROOT_DATABASES

if [[ -n ${INCLUDE[mysqld]} && -f $CONFIGFILE ]] ; then
  if MYSQLD_CONFIGFILE=$(absolute_path "${MYPACKAGES[mysqld]}" $CONFIGFILE) ; then
    ECHO $LINENO mysqld=$MYSQLD_CONFIGFILE
  fi
  for key in MYSQL_PASS MY_CNF ; do
    RESULT=
    [[ $key = MY_CNF ]] && confkey=my-cnf || confkey=mysqld-root-password
    VALUE=$(get_ini_value "$MYSQLD_CONFIGFILE" $confkey 'general definitions' 2>/dev/null) || \
    VALUE=$(get_ini_value $CONFIGFILE $confkey 'general definitions') || RESULT=1
    [[ $RESULT = 1 ]] || EPAGES_INSTALL[$key]=$VALUE
    eval export INSTALL_EPAGES_$key=$VALUE
  done
fi

export INSTALL_EPAGES_ASPOOLDB=
if [[ -f $MYSQLD_CONFIGFILE ]] ; then
  while read section ; do
    case $section in
      ASPoolDB) INSTALL_EPAGES_ASPOOLDB=1 ; continue ;;
      *\ *) continue ;;
    esac
    if LOGIN=$(get_ini_value "$MYSQLD_CONFIGFILE" login "$section") ; then
     [[ $LOGIN = root ]] || continue
      EPAGES_ROOT_DATABASES="$EPAGES_ROOT_DATABASES $section"
    fi
  done < <($INI_CONFIG -f $MYSQLD_CONFIGFILE -G)
fi
if [[ -n $EPAGES_ROOT_DATABASES ]] ; then
  export EPAGES_ROOT_DATABASES
  EPAGES_ROOT_DATABASES=${EPAGES_ROOT_DATABASES# }
  ECHO $LINENO EPAGES_ROOT_DATABASES=$EPAGES_ROOT_DATABASES
fi

#-----------------------------------------------------------------------
# write /etc/sysconfig/epages6 with values from EPAGES_INSTALL
#-----------------------------------------------------------------------

for key in ${!EPAGES_INSTALL[@]} ; do
  case $key in
    MYSQL_CONNECTION) export INSTALL_EPAGES_MYSQL_CONNECTION=${EPAGES_INSTALL[$key]}
      continue ;;
    MYSQL_HOST) export INSTALL_EPAGES_MYSQL_HOST=${EPAGES_INSTALL[$key]}
      continue ;;
  esac
  ECHO $LINENO $INI_CONFIG -f $SYSCONFIG -k EPAGES_INSTALL_$key -v "${EPAGES_INSTALL[$key]}" -i
  [[ -n $MOCK ]] || $INI_CONFIG -f $SYSCONFIG -k EPAGES_INSTALL_$key -v "${EPAGES_INSTALL[$key]}" -i
done

# show variables defined by this script (starting with INSTALL_EPAGES_)
for key in ${!INSTALL_EPAGES_*} ; do
  ECHO $LINENO $key=$(eval echo '$'$key)
done

########################################################################
# singleserver installation of epages
########################################################################

rm -f /tmp/epages/epages-setup/post.*

if [[ -n $SINGLEHOST ]] ; then
  ECHO $LINENO $INSTALLER -y install epages
  if [[ -z $MOCK ]] ; then
    $INSTALLER -y install epages && RESULT=SUCCESS || RESULT=FAILURE
    if type check_install >/dev/null 2>&1 ; then
      check_install setup $RESULT
    fi
  fi
  INCLUDE=()
fi

########################################################################
# distributed installation of epages
########################################################################

#-----------------------------------------------------------------------
# distributed installation: mongo
#-----------------------------------------------------------------------

if [[ -n ${INCLUDE[mongo]} && $OS = debian ]] ; then
  ECHO $LINENO apt-get -y install mongodb-10gen
  [[ -n $MOCK ]] || apt-get -y install mongodb-10gen
  MONGOD_NAME=mongodb
elif [[ -n ${INCLUDE[mongo]} ]] ; then
  ECHO $LINENO yum -y install mongo-10gen mongo-10gen-server
  [[ -n $MOCK ]] || yum -y install mongo-10gen mongo-10gen-server
  MONGOD_NAME=mongod
fi
unset INCLUDE[mongo]

#-----------------------------------------------------------------------
# distributed installation: nlb
#-----------------------------------------------------------------------

if [[ -n ${INCLUDE[nlb]} ]] ; then
  ECHO $LINENO $INSTALLER -y install nginx
  [[ -n $MOCK ]] || $INSTALLER -y install nginx
fi
unset INCLUDE[nlb]

#-----------------------------------------------------------------------
# check that required packages are already installed/preinstalled
# (also on other hosts)
#-----------------------------------------------------------------------

if [[ -f $CONFIGFILE ]] && type find_required_packages >/dev/null 2>&1 ; then
  find_required_packages "$CONFIGFILE" "$MYSQLD_CONFIGFILE"
fi

unset UNSET_CRON SERVER_CONFIGFILE
export EPAGES_INSTALLATION_MOCK=$MOCK
ECHO $LINENO INCLUDE=$(echo ${INCLUDE[@]} | tr ' ' '\n' | sort)

#-----------------------------------------------------------------------
# find /var/epages/postinstall/*.sh
#-----------------------------------------------------------------------

find_postinstall() {
  local postinstall=$VAR_EPAGES/postinstall/$1.sh
  [[ ! -f $postinstall ]] || { echo $postinstall; return 0; }
  return 1
}

#-----------------------------------------------------------------------
# is_preinstalled
#-----------------------------------------------------------------------

is_preinstalled() {
  [[  -f $CONFIGFILE ]] || return 1
  local VALUE
  VALUE=$(get_ini_value $CONFIGFILE preinstalled 'general definitions') || return 1
  if [[ -z $VALUE || $VALUE = 0 ]] ; then
    return 1
  fi
  VALUE=$(get_ini_value $CONFIGFILE installed_databases 'general definitions') || true
  if [[ -n $VALUE ]] ; then
    for DB in ${VALUE//,/ } ; do
      [[ -f ${EPAGES_CONFIG:-/srv/epages/eproot/Shared/Config}/Database.d/$DB.conf ]] || return 1
    done
  fi
  return 0
}

#-----------------------------------------------------------------------
# install_epages_perl
#-----------------------------------------------------------------------

install_epages_perl() {
  # check if epages-perl/cartridges are installed
  # and return (if so)
  local CARTRIDGE=DE_EPAGES::Database::API::Cartridge
  ${PERL:-/srv/epages/eproot/Perl/bin/perl} -M$CARTRIDGE \
    -e "print $CARTRIDGE->new()->version();" >/dev/null 2>&1 && return

  # install package
  rm -f /tmp/epages/epages-perl/post.*
  ECHO $LINENO $INSTALLER -y install epages-perl
  $INSTALLER -y install epages-perl && RESULT=SUCCESS || RESULT=FAILURE

  # check installation
  type check_install >/dev/null 2>&1 || return
  check_install perl $RESULT
}

#-----------------------------------------------------------------------
# legacy: if /var/epages/preinstall/ is missing then just start the services
#-----------------------------------------------------------------------

update_started_packages() {
  local pkg=$1
  local started=/srv/epages/etc/sysconfig/started
  if [[ ! -d $started ]] ; then
    mkdir -m 755 -p /srv/epages
    install -g epages -o ep_appl -m 755 -d $started
  fi
  touch $started/$pkg
}

find_started_packages() {
  local pkg=$1
  local started=/srv/epages/etc/sysconfig/started

  # do not start if package is already started
  [[ ! -f $started/$pkg ]] || return 1

  case $pkg in
    webconf) return 1 ;;
    mysqld) return 0 ;;
    j) [[ -f $started/mysqld ]] && return 0 || return 1 ;;
    reqsrv) [[ -f $started/mysqld ]] && return 0 || return 1 ;;
    ascsrv) [[ -f $started/reqsrv ]] && return 0 || return 1 ;;
    appsrv*) [[ -f $started/${HAS_ASCSRV:-reqsrv} ]] && return 0 || return 1 ;;
  esac
}

start_package() {
  local pkg=$1
  if [[ -z $MOCK && ! $2 =~ /standby ]] && find_started_packages $pkg ; then
    case $pkg in
      appsrv1) /etc/init.d/epages6 start_service || continue ;; # patchhost
      appsrv) IGNORE_INI=1 /etc/init.d/epages6 start_service || continue ;;
      ascsrv) /etc/init.d/epages6 start_asc || continue ;;
      mysqld) /etc/init.d/epages6 start_db || continue ;;
      reqsrv) /etc/init.d/epages6 start_rr || continue ;;
      j) for i in $MONGOD_NAME epages-solr epages-solr-slave epagesj ; do
           [[ -x /etc/init.d/$i ]] || continue
           /etc/init.d/$i start || continue
         done ;;
    esac
  fi
  update_started_packages $pkg
}

create_missing_configfiles() {
  local CONFD=${EPAGES_CONFIG:-/srv/epages/eproot/Shared/Config}
  for suffix in ept initial ; do
    for i in $(ls -d $CONFD/*.$suffix 2>/dev/null) ; do
      [[ ! -e ${i%.$suffix} ]] || continue
      ECHO $LINENO cp -p $i ${i%.$suffix}
      cp -p $i ${i%.$suffix}
    done
  done
}

#-----------------------------------------------------------------------
# distributed installation: epages
#-----------------------------------------------------------------------

unset PREINSTALLED
export IGNORE_INI=
export EPAGES_SPEC_SKIP_RESTART=
export EPAGES_SPEC_KEEP_CONFIG=
[[ -f $CONFIGFILE ]] && KEEP_SHARED_CONFIG=1 || KEEP_SHARED_CONFIG=
export EPAGES_SPEC_KEEP_SHARED_CONFIG=$KEEP_SHARED_CONFIG
EPAGES_CONF_NEW=$(absolute_path "$EPAGES_CONF_NEW" $CONFIGFILE) || true

for package in mysqld webconf reqsrv ascsrv j appsrv ; do
  [[ -n ${INCLUDE[$package]} ]] || continue

  # run all preinstall script
  if [[ -z $PREINSTALLED && $package != j ]] && ! is_preinstalled ; then
    [[ -n $MOCK ]] || { install_epages_perl; create_missing_configfiles; }
    # create configuration files in Shared/Config
    # by running preinstall scripts (.sh or .pl)
    for preinstall_script in $(ls -d $VAR_EPAGES/preinstall/*.?? 2>/dev/null) ; do
      [[ -f $preinstall_script ]] || continue
      ECHO $LINENO $preinstall_script $CONFIGFILE
      chmod 755 $preinstall_script
      [[ -n $MOCK ]] || $preinstall_script ${CONFIGFILE:-/proc/missing/file}
    done
    PREINSTALLED=1
  fi

  # ignore setup for appsrv which is not patchhost
  [[ $package = appsrv && -z $PATCHHOST ]] && IGNORE_INI=1 || IGNORE_INI=

  SERVER_CONFIGFILE=$(absolute_path "${MYPACKAGES[$package]}" $CONFIGFILE) || true

  # decide whether or not to start service when running yum/apt-get install
  [[ $package = j || $SERVER_CONFIGFILE =~ /standby ]] && EPAGES_SPEC_SKIP_RESTART=1 || EPAGES_SPEC_SKIP_RESTART=

  # decide whether to change epages.conf or not
  if [[ $package = webconf ]] ; then
    CHANGE_EPAGES_CONF=1
    if [[ -f $EPAGES_CONF_NEW ]] ; then
      [[ -z $($INI_CONFIG -f "$EPAGES_CONF_NEW" -s 'DE_EPAGES::Object' -k SystemDomainName -G) ]] \
	|| CHANGE_EPAGES_CONF=
      [[ -z $($INI_CONFIG -f "$EPAGES_CONF_NEW" -s 'DE_EPAGES::Store' -k DefaultWebServiceProxy -G) ]] \
	|| CHANGE_EPAGES_CONF=
    fi
    [[ -z $CHANGE_EPAGES_CONF ]] || EPAGES_SPEC_KEEP_SHARED_CONFIG=
  else
    EPAGES_SPEC_KEEP_SHARED_CONFIG=$KEEP_SHARED_CONFIG
  fi

  # install package
  ECHO $LINENO EPAGES_SPEC_KEEP_SHARED_CONFIG=$EPAGES_SPEC_KEEP_SHARED_CONFIG
  ECHO $LINENO EPAGES_SPEC_SKIP_RESTART=$EPAGES_SPEC_SKIP_RESTART
  ECHO $LINENO IGNORE_INI=$IGNORE_INI $INSTALLER -y install epages-$package
  if [[ -z $MOCK ]] ; then
    rm -f /tmp/epages/epages-$package/post.*
    $INSTALLER -y install epages-$package && RESULT=SUCCESS || RESULT=FAILURE
  fi

  # check installation
  if [[ -z $MOCK ]] && type check_install >/dev/null 2>&1 ; then
    check_install $package $RESULT
  fi

  # ignore epages schedulers if appsrv is not installed
  if [[ -z $MOCK && -z ${INCLUDE[appsrv]} && -z $UNSET_CRON ]] ; then
    UNSET_CRON=1
    ECHO $LINENO $INI_CONFIG -f /etc/default/epages6 -k EPAGES_IGNORE_CRON -v 1 -i
    $INI_CONFIG -f /etc/default/epages6 -k EPAGES_IGNORE_CRON -v 1 -i
  fi

  # start package (legacy)
  [[ -d $VAR_EPAGES/postinstall ]] || start_package $package$PATCHHOST "$SERVER_CONFIGFILE"

  # run postinstall script for package
  POSTINSTALL=$(find_postinstall $package) || continue

  # ignore postinstall for appsrv which is not patchhost
  [[ $package$PATCHHOST != appsrv ]] || continue

  # use epages-install.conf for patchhost
  [[ $package = appsrv && ! $SERVER_CONFIGFILE =~ /standby ]] && SERVER_CONFIGFILE=$CONFIGFILE || true

  ECHO $LINENO $POSTINSTALL "$SERVER_CONFIGFILE"
  [[ -n $MOCK ]] || $POSTINSTALL "$SERVER_CONFIGFILE"
done

########################################################################
# update install-epages.conf
########################################################################

#-----------------------------------------------------------------------
# update epages repository
#-----------------------------------------------------------------------

if  [[ $OS = redhat ]] && type enable_repo >/dev/null ; then
  enable_repo -e 0 setup >/dev/null || true
fi
if [[ $OS = debian ]] && type enable_apt_repo >/dev/null ; then
  enable_apt_repo 0 epages-setup >/dev/null || true
fi

#-----------------------------------------------------------------------
# touch $CONFIGFILE
#-----------------------------------------------------------------------

# use default config file unless defined
CONFIGFILE=${CONFIGFILE:-/srv/epages/etc/sysconfig/install-epages.conf}
ECHO $LINENO touch $CONFIGFILE
mkdir -m 755 -p ${CONFIGFILE%/*}
touch $CONFIGFILE
chmod 600 $CONFIGFILE

#-----------------------------------------------------------------------
# update preinstalled in install-epages.conf
#-----------------------------------------------------------------------

if [[ -n $PREINSTALLED ]] ; then
  ECHO $LINENO $INI_CONFIG -f $CONFIGFILE -s 'general definitions' -k preinstalled -v $PREINSTALLED -i
  $INI_CONFIG -f $CONFIGFILE -s 'general definitions' -k preinstalled -v $PREINSTALLED -i
fi

#-----------------------------------------------------------------------
# update groups/users in install-epages.conf
#-----------------------------------------------------------------------

for group in epages ep_web ; do
  gid=$(get_gid $group) || continue
  ECHO $LINENO $INI_CONFIG -f $CONFIGFILE -s 'unix groups' -k $group -v "$gid" -i
  $INI_CONFIG -f $CONFIGFILE -s 'unix groups' -k $group -v "$gid" -i
done

for user in eprunapp epagesj ep_appl ep_web eprunweb ; do
  uid=$(get_uid $user) || continue
  ECHO $LINENO $INI_CONFIG -f $CONFIGFILE -s 'unix users' -k $user -v "$uid" -i
  $INI_CONFIG -f $CONFIGFILE -s 'unix users' -k $user -v "$uid" -i
done

#-----------------------------------------------------------------------
# update package information in install-epages.conf
#-----------------------------------------------------------------------

if [[ -f $CONFIGFILE ]] && type update_required_packages >/dev/null 2>&1 ; then
  update_required_packages "$CONFIGFILE" "$MYSQLD_CONFIGFILE"
fi

#-----------------------------------------------------------------------
# exit
#-----------------------------------------------------------------------

exit 0
